@@ -52,7 +52,7 @@ class DefaultRequirements(SuiteRequirements):
         """Target database must also enforce check constraints."""
 
         return self.check_constraints + fails_on(
-            self._mysql_not_mariadb_102,
+            self._mysql_check_constraints_dont_exist,
             "check constraints don't enforce on MySQL, MariaDB<10.2",
         )
 
@@ -167,7 +167,7 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def no_quoting_special_bind_names(self):
-        """Target database will quote bound paramter names, doesn't support
+        """Target database will quote bound parameter names, doesn't support
         EXPANDING"""
 
         return skip_if(["oracle"])
@@ -349,6 +349,10 @@ class DefaultRequirements(SuiteRequirements):
             "postgresql", "doesn't support sequences as a server side default."
         )
 
+    @property
+    def sql_expressions_inserted_as_primary_key(self):
+        return only_if([self.returning, self.sqlite])
+
     @property
     def correlated_outer_joins(self):
         """Target must support an outer join to a subquery which
@@ -453,7 +457,10 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def check_constraint_reflection(self):
         return fails_on_everything_except(
-            "postgresql", "sqlite", "oracle", self._mariadb_102
+            "postgresql",
+            "sqlite",
+            "oracle",
+            self._mysql_and_check_constraints_exist,
         )
 
     @property
@@ -596,7 +603,7 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def window_functions(self):
         return only_if(
-            ["postgresql>=8.4", "mssql", "oracle"],
+            ["postgresql>=8.4", "mssql", "oracle", "sqlite>=3.25.0"],
             "Backend does not support window functions",
         )
 
@@ -662,6 +669,16 @@ class DefaultRequirements(SuiteRequirements):
 
         return exclusions.open()
 
+    @property
+    def expressions_against_unbounded_text(self):
+        """target database supports use of an unbounded textual field in a
+        WHERE clause."""
+
+        return fails_if(
+            ["oracle"],
+            "ORA-00932: inconsistent datatypes: expected - got CLOB",
+        )
+
     @property
     def unicode_data(self):
         """target drive must support unicode data stored in columns."""
@@ -770,6 +787,7 @@ class DefaultRequirements(SuiteRequirements):
                     )
                 ),
                 "postgresql >= 9.3",
+                "sqlite >= 3.9",
             ]
         )
 
@@ -780,6 +798,7 @@ class DefaultRequirements(SuiteRequirements):
                 lambda config: against(config, "mysql >= 5.7")
                 and not config.db.dialect._is_mariadb,
                 "postgresql >= 9.3",
+                "sqlite >= 3.9",
             ]
         )
 
@@ -1001,6 +1020,44 @@ class DefaultRequirements(SuiteRequirements):
             ]
         )
 
+    @property
+    def implicit_decimal_binds(self):
+        """target backend will return a selected Decimal as a Decimal, not
+        a string.
+
+        e.g.::
+
+            expr = decimal.Decimal("15.7563")
+
+            value = e.scalar(
+                select([literal(expr)])
+            )
+
+            assert value == expr
+
+        See :ticket:`4036`
+
+        """
+
+        # fixed for mysqlclient in
+        # https://github.com/PyMySQL/mysqlclient-python/commit/68b9662918577fc05be9610ef4824a00f2b051b0
+        def check(config):
+            if against(config, "mysql+mysqldb"):
+                # can remove once post 1.3.13 is released
+                try:
+                    from MySQLdb import converters
+                    from decimal import Decimal
+
+                    return Decimal not in converters.conversions
+                except:
+                    return True
+
+            return against(
+                config, "mysql+mysqldb"
+            ) and config.db.dialect._mysql_dbapi_version <= (1, 3, 13)
+
+        return exclusions.fails_on(check, "fixed for mysqlclient post 1.3.13")
+
     @property
     def fetch_null_from_numeric(self):
         return skip_if(("mssql+pyodbc", None, None, "crashes due to bug #351"))
@@ -1133,9 +1190,9 @@ class DefaultRequirements(SuiteRequirements):
         lookup = {
             # will raise without quoting
             "postgresql": "POSIX",
-            # note MySQL databases need to be created w/ utf8mb3 charset
+            # note MySQL databases need to be created w/ utf8mb4 charset
             # for the test suite
-            "mysql": "utf8mb3_bin",
+            "mysql": "utf8mb4_bin",
             "sqlite": "NOCASE",
             # will raise *with* quoting
             "mssql": "Latin1_General_CI_AS",
@@ -1169,6 +1226,25 @@ class DefaultRequirements(SuiteRequirements):
     def no_mssql_freetds(self):
         return self.mssql_freetds.not_()
 
+    @property
+    def pyodbc_fast_executemany(self):
+        def has_fastexecutemany(config):
+            if not against(config, "mssql+pyodbc"):
+                return False
+            if config.db.dialect._dbapi_version() < (4, 0, 19):
+                return False
+            with config.db.connect() as conn:
+                drivername = conn.connection.connection.getinfo(
+                    config.db.dialect.dbapi.SQL_DRIVER_NAME
+                )
+                # on linux this is 'libmsodbcsql-13.1.so.9.2'.
+                # don't know what it is on windows
+                return "msodbc" in drivername
+
+        return only_if(
+            has_fastexecutemany, "only on pyodbc > 4.0.19 w/ msodbc driver"
+        )
+
     @property
     def python_fixed_issue_8743(self):
         return exclusions.skip_if(
@@ -1220,6 +1296,17 @@ class DefaultRequirements(SuiteRequirements):
 
         return only_if(check)
 
+    @property
+    def mysql_ngram_fulltext(self):
+        def check(config):
+            return (
+                against(config, "mysql")
+                and not config.db.dialect._is_mariadb
+                and config.db.dialect.server_version_info >= (5, 7)
+            )
+
+        return only_if(check)
+
     def _mariadb_102(self, config):
         return (
             against(config, "mysql")
@@ -1227,6 +1314,33 @@ class DefaultRequirements(SuiteRequirements):
             and config.db.dialect._mariadb_normalized_version_info > (10, 2)
         )
 
+    def _mysql_and_check_constraints_exist(self, config):
+        # 1. we have mysql / mariadb and
+        # 2. it enforces check constraints
+        if exclusions.against(config, "mysql"):
+            if config.db.dialect._is_mariadb:
+                norm_version_info = (
+                    config.db.dialect._mariadb_normalized_version_info
+                )
+                return norm_version_info >= (10, 2)
+            else:
+                norm_version_info = config.db.dialect.server_version_info
+                return norm_version_info >= (8, 0, 16)
+        else:
+            return False
+
+    def _mysql_check_constraints_exist(self, config):
+        # 1. we dont have mysql / mariadb or
+        # 2. we have mysql / mariadb that enforces check constraints
+        return not exclusions.against(
+            config, "mysql"
+        ) or self._mysql_and_check_constraints_exist(config)
+
+    def _mysql_check_constraints_dont_exist(self, config):
+        # 1. we have mysql / mariadb and
+        # 2. they dont enforce check constraints
+        return not self._mysql_check_constraints_exist(config)
+
     def _mysql_not_mariadb_102(self, config):
         return against(config, "mysql") and (
             not config.db.dialect._is_mariadb
@@ -1258,6 +1372,13 @@ class DefaultRequirements(SuiteRequirements):
             and config.db.scalar("show server_encoding").lower() == "utf8"
         )
 
+    @property
+    def cxoracle6_or_greater(self):
+        return only_if(
+            lambda config: against(config, "oracle+cx_oracle")
+            and config.db.dialect.cx_oracle_ver >= (6,)
+        )
+
     @property
     def oracle5x(self):
         return only_if(
